﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;


namespace TerritoriesWithFixedAllocations
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        /**
         * This sample will use PTV xTerritory, to distribute a set of locations amongst 3 predefined territory centers.
         * Subsequently PTV xMap will be used to visualize the territories and the fixed allocations.
        **/ 
        private void runSample()
        {
            Territories territories = calculateTerritories();
            if (territories != null)
                mapTerritories(territories);
        }

        private Territories calculateTerritories()
        {
            //Step 1: Initiate xTerritory client
            XTerritoryWSService xTerritoryClient = new XTerritoryWSService();

            //Step 2: Create CallerContext with appropriate profile and coordinate format
            CallerContext context = new CallerContext();
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            context.wrappedProperties = properties;

            //Step 3: Load the locations for territory planning
            List<String> locationStrings = getLocations();
            Location[] locations = new Location[locationStrings.Count - 3];
            Territory[] initTerritories = new Territory[3];
            int territoryCounter = 0;
            int locationCounter = 0;
            foreach (String s in locationStrings)
            {
                String[] splitString = s.Split(';');
                
                //Step 3.1: Load location ID, x and y coordinates, activity measure and the identifier for center/ fixed allocations
                String locID = splitString[0];
                double locX = Convert.ToDouble(splitString[1]);
                double locY = Convert.ToDouble(splitString[2]);
                String identifier = splitString[4];
                double locActivity = 0.0;
                if (identifier != "C")
                    locActivity = Convert.ToDouble(splitString[3]);
                
                xserver.Point point = new xserver.Point
                {
                    point = new PlainPoint { x = locX, y = locY }
                };

                //Step 3.2: Create territory centers, if the input data is marked as a center
                if (identifier == "C")
                {
                    initTerritories[territoryCounter] = new Territory
                    {
                        id = locID,
                        referencePoint = point
                    };
                    territoryCounter++;
                }
                //Step 3.3: If the identifier specifies one, create a location with a fixed allocation
                else if (identifier != "")
                {
                    locations[locationCounter] = new Location
                    {
                        id = locID,
                        coordinate = point,
                        activity = new AbsoluteActivity { value = locActivity },
                        fixedToTerritorySpecified = true,
                        fixedToTerritory = true,
                        territoryId = identifier
                    };
                    locationCounter++;
                }
                //Step 3.4: Otherwise create a location, that is not fixed to a specific territory
                else
                {
                    locations[locationCounter] = new Location
                    {
                        id = locID,
                        coordinate = point,
                        activity = new AbsoluteActivity { value = locActivity },
                    };
                    locationCounter++;
                }
            }


            //Step 4: Create DistanceCalculationOptions
            NewDistanceMatrixByRoad distOptions = new NewDistanceMatrixByRoad();

            //Step 5: Create PlanningProfile
            PlanningProfile profile = new PlanningProfile
            {
                solutionQuality = SolutionQuality.BASIC,    //Solution quality, with larger problems this impact the running time severely
                avoidOverlappingTerritories = true,         //If true, the algorithm tries to avoid visual overlaps between territories,
                                                            //if a solution without overlaps is possible
                createBalancedTerritories = false,          //If true, the imbalanceTolerance parameter will be considered
                imbalanceTolerance = 10.0,                  //Maximum deviation from average activity per territory
            };

            //Step 6: Create ResponseContents, specify which information the client returns
            ResponseContents contents = new ResponseContents
            {
                locations = true,
                territories = true,
            };

            try
            {
                //Step 7: Calculate territories
                Territories territories = xTerritoryClient.distributeLocations(locations, initTerritories, distOptions, profile, contents, context);
                return territories;
            }
            catch (Exception e)
            {
                return null;
            }
        }

        private void mapTerritories(Territories territories)
        {
            //Step 1: Create xMap client
            XMapWSService client = new XMapWSService();

            //Step 2: Create ImageInfo
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 800,
                width = 1280,
                imageParameter = ""
            };

            //Step 3: Create MapParams
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

            //Step 4.1: Create Bitmap array for locations, each location will be visualized with a bitmap marker
            Bitmap[] bitmapArrayLocations = new Bitmap[territories.wrappedLocations.Length];
            
            //Step 4.2: Create Bitmap array for territory centers, each center will be visualized with a bitmap marker
            Bitmap[] bitmapArrayTerritories = new Bitmap[territories.wrappedTerritories.Length];
            
            int locationCounter = 0;
            int territoryCounter = 0;

            //Step 5.1: Create list of territoryIDs, it will be used to match with the territoryIDs of the locations
            List<String> territoryIDs = new List<String>();
            
            //Step 5.2: Create a dictionary, to access the territories by their ID
            Dictionary<String, Territory> territoryDict = new Dictionary<String, Territory>();
            
            //Step 6: Fill list and dictionary, add territory centers to bitmap array
            foreach (Territory ter in territories.wrappedTerritories)
            {
                territoryIDs.Add(ter.id);
                territoryDict.Add(ter.id, ter);
                String bmp = "dealer.bmp";
                xserver.Point point = ter.referencePoint;
                bitmapArrayTerritories[territoryCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };
                territoryCounter++;
            }

            //Step 7: Create list for lineLayers, we will use lines to visualize, which locations are fixed to a certain territory center
            List<CustomLayer> lineLayers = new List<CustomLayer>();
            
            foreach (Location location in territories.wrappedLocations)
            {
                //Step 8: Add location to bitmap array
                String bmp = "location.bmp";
                if (location.territoryId == territoryIDs[0])
                {
                    bmp = "flaggreen.bmp";
                }
                else if (location.territoryId == territoryIDs[1])
                {
                    bmp = "flagred.bmp";
                }
                else if (location.territoryId == territoryIDs[2])
                {
                    bmp = "flagblue.bmp";
                }
                xserver.Point point = location.coordinate;
                bitmapArrayLocations[locationCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };

                //Step 9: If the location is fixed to a territory, add a line, that connects the location with the center of the territory
                if (location.fixedToTerritory == true)
                {
                    //Step 9.1: Add territory center and location to PlainPoint array
                    PlainPoint[] plainPoints = new PlainPoint[2];
                    plainPoints[0] = location.coordinate.point;
                    plainPoints[1] = territoryDict[location.territoryId].referencePoint.point;

                    // Step 9.2: Create PLainLineString and add it to a LineString array
                    PlainLineString plainLineString = new PlainLineString { wrappedPoints = plainPoints };
                    LineString[] lineStrings = new LineString[]
                    {
                        new LineString { lineString = plainLineString }
                    };

                    //Step 9.3: Specify LineOptions depending on the territoryID
                    LineOptions lineOptions = new LineOptions();
                    xserver.Color color = new xserver.Color { red = 0, green = 255, blue = 0 };
                    if (location.territoryId == territoryIDs[1])
                        color = new xserver.Color { red = 255, green = 0, blue = 0 };
                    if (location.territoryId == territoryIDs[2])
                        color = new xserver.Color { red = 0, green = 0, blue = 255 };
                    LinePartOptions partoptions = new LinePartOptions
                    {
                        visible = true,
                        color = color,
                        width = 5,
                    };
                    lineOptions.mainLine = partoptions;

                    //Step 9.4: Create Lines array
                    Lines[] lines = new Lines[1];
                    lines[0] = new Lines { wrappedLines = lineStrings, options = lineOptions };

                    //Steo 9.5: Create CustomLayer and add it to the list
                    CustomLayer customLineLayer = new CustomLayer
                    {
                        visible = true,
                        drawPriority = 25000,
                        wrappedLines = lines,
                        objectInfos = ObjectInfoType.NONE
                    };
                    lineLayers.Add(customLineLayer);
                }
                locationCounter++;
            }

            //Step 10: Create BitmapOptions and Bitmaps arrays (one for territories, one for locations)
            BitmapOptions bitmapOptionsLocations = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 25, y = 32 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            BitmapOptions bitmapOptionsTerritories = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 12, y = 12 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            Bitmaps[] bitmaps = new Bitmaps[] 
            { 
                new Bitmaps { wrappedBitmaps = bitmapArrayLocations, options = bitmapOptionsLocations },
                new Bitmaps { wrappedBitmaps = bitmapArrayTerritories, options = bitmapOptionsTerritories }
            };

            //Step 11: Create CustomLayer for bitmaps
            CustomLayer customBMPLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25001,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            //Step 12: Add all CustomLayers (bitmaps and lines) to a Layer array
            Layer[] layers = new Layer[lineLayers.Count + 1];
            layers[0] = customBMPLayer;
            for (int i = 1; i < layers.Length; i++)
            {
                layers[i] = lineLayers[i - 1];
            }

            bool includeImageInResponse = false;

            try
            {
                // Step 13: Return map object with its characteristics
                Map map = client.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 14: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // proper exception handling: see API for PTV's exception handling!
            }
        }

        private List<String> getLocations()
        {
            List<String> list = new List<String>();

            // orderId, X, Y, activity, identifier (specifies if a location is a territory center or fixed to one, if this value is "" neither is the case)
            list.Add("101;671006;6382294;;C");
            list.Add("102;681812;6375071;120;");
            list.Add("103;682676;6388154;90;");
            list.Add("104;675000;6380000;60;");
            list.Add("105;679143;6374455;90;");
            list.Add("106;684621;6374703;180;");
            list.Add("107;684659;6372947;90;");
            list.Add("108;683151;6371248;30;");
            list.Add("109;685669;6371954;45;");
            list.Add("110;683974;6369193;60;");
            list.Add("111;683000;6366903;90;113");
            list.Add("112;678763;6367800;120;");
            list.Add("113;681397;6369670;;C");
            list.Add("114;678400;6370854;90;");
            list.Add("115;674779;6373188;60;101");
            list.Add("116;678149;6380180;45;");
            list.Add("117;680440;6381326;30;113");
            list.Add("118;665627;6378195;90;101");
            list.Add("119;687472;6386122;180;");
            list.Add("120;666773;6381020;120;");
            list.Add("121;687136;6382563;60;");
            list.Add("122;665642;6382471;30;");
            list.Add("123;665627;6386365;45;101");
            list.Add("124;672957;6388121;60;");
            list.Add("125;677004;6388121;75;126");
            list.Add("126;682578;6384074;;C");
            list.Add("127;687540;6379111;105;126");
            list.Add("128;681661;6384838;120;");
            list.Add("129;687388;6385754;135;");
            list.Add("130;685937;6388121;150;");

            return list;
        }
    }
}
