﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace TerritoryPlanningWithFixedCenters
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        /**
         * This sample will use PTV xTerritory, to distribute a set of locations amongst 3 predefined territory centers.
         * Subsequently PTV xMap will be used to visualize the territories.
        **/
        private void runSample()
        {
            Territories territories = calculateTerritories();
            if (territories != null)
                mapTerritories(territories);
        }

        private Territories calculateTerritories()
        {
            //Step 1: Initiate xTerritory client
            XTerritoryWSService xTerritoryClient = new XTerritoryWSService();

            //Step 2: Create CallerContext with appropriate profile and coordinate format
            CallerContext context = new CallerContext();
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            context.wrappedProperties = properties;

            //Step 3: Load the locations for territory planning
            List<String> locationStrings = getLocations();
            Location[] locations = new Location[locationStrings.Count - 3];
            Territory[] initTerritories = new Territory[3];
            int territoryCounter = 0;
            int locationCounter = 0;
            foreach (String s in locationStrings)
            {
                String[] splitString = s.Split(';');

                //Step 3.1: Load location ID, x and y coordinates, activity measure and the identifier for teritory centers
                String locID = splitString[0];
                double locX = Convert.ToDouble(splitString[1]);
                double locY = Convert.ToDouble(splitString[2]);
                String identifier = splitString[4];
                double locActivity = 0.0;
                if (identifier != "C")
                    locActivity = Convert.ToDouble(splitString[3]);
                
                xserver.Point point = new xserver.Point
                {
                    point = new PlainPoint { x = locX, y = locY }
                };

                //Step 3.3: Create territory, if the input data is marked as a territory center
                if (identifier == "C")
                {
                    initTerritories[territoryCounter] = new Territory
                    {
                        id = locID,
                        referencePoint = point
                    };
                    territoryCounter++;
                }
                //Step 3.3: Otherwise create a location, with the data above         
                else
                {
                    locations[locationCounter] = new Location
                    {
                        id = locID,
                        coordinate = point,
                        activity = new AbsoluteActivity { value = locActivity },
                    };
                    locationCounter++;
                }
            }


            //Step 4: Create DistanceCalculationOptions
            NewDistanceMatrixByRoad distOptions = new NewDistanceMatrixByRoad();

            //Step 5: Create PlanningProfile
            PlanningProfile profile = new PlanningProfile
            {
                solutionQuality = SolutionQuality.BASIC,    //Solution quality, with larger problems this impact the running time severely
                avoidOverlappingTerritories = true,         //If true, the algorithm tries to avoid visual overlaps between territories,
                                                            //if a solution without overlaps is possible
                createBalancedTerritories = false,          //If true, the imbalanceTolerance parameter will be considered
                imbalanceTolerance = 10.0,                  //Maximum deviation from average activity per territory
            };

            //Step 6: Create ResponseContents, specify which information the client returns
            ResponseContents contents = new ResponseContents
            {
                locations = true,
                territories = true,
            };

            try
            {
                //Step 7: Calculate territories
                Territories territories = xTerritoryClient.distributeLocations(locations, initTerritories, distOptions, profile, contents, context);
                return territories;
            }
            catch (Exception e)
            {
                return null;
            }
        }

        private void mapTerritories(Territories territories)
        {
            //Step 1: Create xMap client
            XMapWSService client = new XMapWSService();

            //Step 2: Create ImageInfo
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 800,
                width = 1280,
                imageParameter = ""
            };

            //Step 3: Create MapParams
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

            //Step 4.1: Create Bitmap array for locations, each location will be visualized with a bitmap marker
            Bitmap[] bitmapArrayLocations = new Bitmap[territories.wrappedLocations.Length];
            
            //Step 4.2: Create Bitmap array for territory centers, each center will be visualized with a bitmap marker
            Bitmap[] bitmapArrayTerritories = new Bitmap[territories.wrappedTerritories.Length];
            
            int locationCounter = 0;
            int territoryCounter = 0;

            //Step 5: Create list of territoryIDs, it will be used to match with the territoryIDs of the locations
            List<String> territoryIDs = new List<String>();

            //Step 6: Fill list and add territory centers to bitmap array
            foreach (Territory ter in territories.wrappedTerritories)
            {
                territoryIDs.Add(ter.id);
                String bmp = "dealer.bmp";
                xserver.Point point = ter.referencePoint;
                bitmapArrayTerritories[territoryCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };
                territoryCounter++;
            }

            //Step 7: Add locations to bitmap array
            foreach (Location location in territories.wrappedLocations)
            {
                String bmp = "location.bmp";
                if (location.territoryId == territoryIDs[0])
                {
                    bmp = "flaggreen.bmp";
                }
                else if (location.territoryId == territoryIDs[1])
                {
                    bmp = "flagred.bmp";
                }
                else if (location.territoryId == territoryIDs[2])
                {
                    bmp = "flagblue.bmp";
                }
                xserver.Point point = location.coordinate;
                bitmapArrayLocations[locationCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };

                locationCounter++;
            }

            //Step 8: Create BitmapOptions and Bitmaps arrays (one for territories, one for locations)
            BitmapOptions bitmapOptionsLocations = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 25, y = 32 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            BitmapOptions bitmapOptionsTerritories = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 12, y = 12 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            Bitmaps[] bitmaps = new Bitmaps[] 
            { 
                new Bitmaps { wrappedBitmaps = bitmapArrayLocations, options = bitmapOptionsLocations },
                new Bitmaps { wrappedBitmaps = bitmapArrayTerritories, options = bitmapOptionsTerritories }
            };

            //Step 9: Create CustomLayer for bitmaps
            CustomLayer customBMPLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25001,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            //Step 10: Add custom layer to layer array
            Layer[] layers = new Layer[1];
            layers[0] = customBMPLayer;

            bool includeImageInResponse = false;

            try
            {
                // Step 11: Return map object with its characteristics
                Map map = client.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 12: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // proper exception handling: see API for PTV's exception handling!
            }
        }

        private List<String> getLocations()
        {
            List<String> list = new List<String>();

            // orderId, X, Y, activity, identifier (specifies if a location is a territory center)
            list.Add("101;671006;6382294;;C");
            list.Add("102;681812;6375071;120;");
            list.Add("103;682676;6388154;90;");
            list.Add("104;675000;6380000;60;");
            list.Add("105;679143;6374455;90;");
            list.Add("106;684621;6374703;180;");
            list.Add("107;684659;6372947;90;");
            list.Add("108;683151;6371248;30;");
            list.Add("109;685669;6371954;45;");
            list.Add("110;683974;6369193;60;");
            list.Add("111;683000;6366903;90;");
            list.Add("112;678763;6367800;120;");
            list.Add("113;681397;6369670;;C");
            list.Add("114;678400;6370854;90;");
            list.Add("115;674779;6373188;60;");
            list.Add("116;678149;6380180;45;");
            list.Add("117;680440;6381326;30;");
            list.Add("118;665627;6378195;90;");
            list.Add("119;687472;6386122;180;");
            list.Add("120;666773;6381020;120;");
            list.Add("121;687136;6382563;60;");
            list.Add("122;665642;6382471;30;");
            list.Add("123;665627;6386365;45;");
            list.Add("124;672957;6388121;60;");
            list.Add("125;677004;6388121;75;");
            list.Add("126;682578;6384074;;C");
            list.Add("127;687540;6379111;105;");
            list.Add("128;681661;6384838;120;");
            list.Add("129;687388;6385754;135;");
            list.Add("130;685937;6388121;150;");

            return list;
        }
    }
}
