﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace TerritoryPlanningWithGroups
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        /**
         * This sample will use PTV xTerritory, to calculate 3 territories for a set of locations with one activity measure.
         * Locations will be grouped by the first 2 digits of their postcode and locations in the same group will be assigned to the same territory.
         * The territories will subsequently be visualized, using PTV xMap.
        **/
        private void runSample()
        {
            Territories territories = calculateTerritories();
            if (territories != null)
                mapTerritories(ref territories);
        }

        private Territories calculateTerritories()
        {
            //Step 1: Initiate xTerritory client
            XTerritoryWSService xTerritoryClient = new XTerritoryWSService();

            //Step 2: Create CallerContext with appropriate profile and coordinate format
            CallerContext context = new CallerContext();
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            context.wrappedProperties = properties;

            //Step 3: Load the locations for territory planning
            List<String> locationStrings = getLocations();
            Location[] locations = new Location[locationStrings.Count + 1];
            for (int i = 0; i < locationStrings.Count; i++)
            {
                String[] splitString = locationStrings[i].Split(';');
                String locID = splitString[0];

                //Step 3.1: Load x and y coordinates, activity measure and postcode
                double locX = Convert.ToDouble(splitString[1]);
                double locY = Convert.ToDouble(splitString[2]);
                double locActivity = Convert.ToDouble(splitString[3]);
                String plz = splitString[4];
                plz = plz.Substring(0, 2);              //Take first two digits of the poscode, in Germany these identify a postcod region

                xserver.Point point = new xserver.Point
                {
                    point = new PlainPoint { x = locX, y = locY }
                };

                //Step 3.2: Create a Location with the data above
                locations[i] = new Location
                {
                    id = locID,
                    coordinate = point,
                    activity = new AbsoluteActivity { value = locActivity },
                    properties = new LocationProperties { group = plz }
                };

                //Please ignore (circumvents a bug, where no territories are returned, if all locations are assigned to a group)
                if (i == 0)
                {
                    locations[locations.Length - 1] = new Location
                    {
                        id = "dummy",
                        coordinate = point,
                        activity = new AbsoluteActivity { value = 0 },
                    };
                }
            }

            //Step 4: Create DistanceCalculationOptions, here direct distances will be used as an approximation
            ApproximateByDirectDistance distOptions = new ApproximateByDirectDistance();

            //Step 5: Create PlanningOptions and specify the number of territories
            PlanningOptions options = new PlanningOptions
            {
                exactNumberOfTerritories = 3                //the visualization part of this sample supports a maximum of 3 territories
            };

            //Step 6: Create PlanningProfile
            PlanningProfile profile = new PlanningProfile
            {
                solutionQuality = SolutionQuality.BASIC,    //Solution quality, with larger problems this impacts the running time severely
                avoidOverlappingTerritories = true,         //If true, the algorithm tries to avoid visual overlaps between territories,
                                                            //if a solution without overlaps is possible
                createBalancedTerritories = true,           //If true, the imbalanceTolerance parameter will be considered
                imbalanceTolerance = 10.0,                  //Maximum deviation from average activity per territory
            };

            //Step 7: Create ResponseContents, specify which information the client returns
            ResponseContents contents = new ResponseContents
            {
                locations = true,
                territories = true,
            };

            try
            {
                //Step 8: Calculate territories
                Territories territories = xTerritoryClient.planTerritories(locations, distOptions, options, profile, contents, context);
                return territories;
            }
            catch (Exception e)
            {
                return null;
            }
        }

        private void mapTerritories(ref Territories territories)
        {
            //Step 1: Create xMap client
            XMapWSService client = new XMapWSService();

            //Step 2: Create ImageInfo
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 800,
                width = 1280,
                imageParameter = ""
            };

            //Step 3: Create MapParams
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

            //Step 4: Create Bitmap array, each location will be visualized with a bitmap marker
            Bitmap[] bitmapArray = new Bitmap[territories.wrappedLocations.Length + 3];

            //Step 5: Create list of territoryIDs, it will be used to match with the territoryIDs of the locations
            List<String> territoryIDs = new List<String>();
            foreach (Territory ter in territories.wrappedTerritories)
            {
                territoryIDs.Add(ter.id);
            }

            //Step 6: Insert appropriate Bitmap into array for each location
            int locationCounter = 0;
            foreach (Location location in territories.wrappedLocations)
            {
                String bmp = "location.bmp";
                if (location.territoryId == territoryIDs[0])
                {
                    bmp = "flaggreen.bmp";
                }
                else if (location.territoryId == territoryIDs[1])
                {
                    bmp = "flagred.bmp";
                }
                else if (location.territoryId == territoryIDs[2])
                {
                    bmp = "flagblue.bmp";
                }
                xserver.Point point = location.coordinate;
                bitmapArray[locationCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };
                locationCounter++;
            }

            //Step 7: Create BitmapOptions and Bitmaps[]
            BitmapOptions bitmapOptions = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 25, y = 32 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };

            //Step 8: Create CustomLayer for bitmaps
            CustomLayer customBMPLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25002,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            Layer[] layers = new Layer[1];
            layers[0] = customBMPLayer;

            bool includeImageInResponse = false;

            try
            {
                // Step 10: Return map object with its characteristics
                Map map = client.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 11: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // proper exception handling: see API for PTV's exception handling!
            }
        }
        
        private List<String> getLocations()
        {
            List<String> list = new List<String>();

            // orderId, X, Y, activity
            list.Add("1;1020545;6229507;89892;70173");
            list.Add("2;1023859;6231238;51561;70188");
            list.Add("3;1032242;6237049;59280;70736");
            list.Add("4;1030486;6236143;88776;70734");
            list.Add("5;1029472;6241035;70749;70736");
            list.Add("6;1043680;6246335;40608;71364");
            list.Add("7;1055232;6302689;80407;74613");
            list.Add("8;932507;6269368;91943;76133");
            list.Add("9;1187619;6335581;27168;90599");
            list.Add("10;1104174;6403372;69992;97070");
            list.Add("11;1100865;6403659;90358;97082");
            list.Add("12;1131011;6392747;49666;97318");
            list.Add("13;1149599;6402669;92899;97353");
            list.Add("14;1153928;6192593;40046;89423");
            list.Add("15;1167419;6307767;29687;91567");
            list.Add("16;1093982;6372182;29560;97957");
            list.Add("17;1047281;6236216;71231;73630");
            list.Add("18;1021974;6438737;48264;63773");
            list.Add("19;1205530;6310702;21659;91575");
            list.Add("20;1158561;6194517;26764;89415");
            list.Add("21;1031535;6435937;15003;63857");
            list.Add("22;1004814;6426934;49998;63762");
            list.Add("23;991792;6416404;34120;64823");
            list.Add("24;1079895;6364222;57677;97922");
            list.Add("25;1068881;6412385;88414;97828");
            list.Add("26;969935;6253410;86934;75177");
            list.Add("27;1137706;6175246;75773;89340");
            list.Add("28;935022;6288899;93400;76351");
            list.Add("29;986652;6310644;28683;74889");
            list.Add("30;1033291;6243257;95262;71334");
            list.Add("31;1046950;6251068;15571;71397");
            list.Add("32;1136994;6415386;28997;97332");
            list.Add("33;1143944;6193166;17984;89567");
            list.Add("34;1191803;6361423;69575;91448");
            list.Add("35;1020502;6473951;40946;63571");
            list.Add("36;929741;6374760;74792;67547");
            list.Add("37;1104439;6403707;95697;97070");
            list.Add("38;995405;6430637;96562;64832");
            list.Add("39;1019679;6433157;16679;63743");
            list.Add("40;1023667;6183840;75604;72760");
            list.Add("41;1107982;6166068;68950;89077");
            list.Add("42;1104280;6403732;49177;97070");
            list.Add("43;941510;6350891;46299;68161");
            list.Add("44;968232;6337914;63395;69117");
            list.Add("45;938019;6321816;71294;67346");
            list.Add("46;902913;6301424;51703;76829");
            list.Add("47;1020502;6230242;46127;70173");
            list.Add("48;966250;6274639;51244;75015");
            list.Add("49;939115;6273994;91322;76139");
            list.Add("50;1085065;6439399;57842;97753");
            list.Add("51;1086614;6352073;64689;97980");
            list.Add("52;1149479;6278930;68672;91550");
            list.Add("53;988341;6441598;95846;63110");
            list.Add("54;1065548;6214563;61076;73066");
            list.Add("55;1062460;6229777;51337;73614");
            list.Add("56;1072258;6222004;16480;73098");
            list.Add("57;1085373;6213673;86256;73079");
            list.Add("58;1043439;6236404;42919;71384");
            list.Add("59;1062061;6409263;35843;97828");
            list.Add("60;1152370;6258167;85840;73485");
            list.Add("61;1078212;6333499;72929;74677");
            list.Add("62;1148701;6278127;92997;91550");
            list.Add("63;1098311;6464145;61497;97762");
            list.Add("64;1064582;6438154;89642;97816");
            list.Add("65;936061;6269301;52026;76137");
            list.Add("66;1014254;6255041;68385;71732");
            list.Add("67;1031665;6246408;68935;71686");
            list.Add("68;1089721;6214010;26233;73072");
            list.Add("69;1082271;6215727;22372;73084");
            list.Add("70;1075461;6217881;20239;73033");
            list.Add("71;1022113;6438765;26434;63773");
            list.Add("72;976418;6442334;55958;63128");
            list.Add("73;1071536;6349663;74599;97944");
            list.Add("74;1092520;6397383;38112;97249");
            list.Add("75;1019501;6216688;20791;70771");
            list.Add("76;934166;6376534;15103;68623");
            list.Add("77;1040952;6212847;30064;73257");
            list.Add("78;987172;6309850;52752;74889");
            list.Add("79;1099061;6409494;43878;97209");
            list.Add("80;1016717;6435990;43594;63741");
            list.Add("81;1115236;6237351;45192;73457");
            list.Add("82;1128611;6394531;88764;97318");
            list.Add("83;960287;6377919;32429;64646");
            list.Add("84;927740;6372999;55610;67547");
            list.Add("85;1198657;6350471;26377;90579");
            list.Add("86;1021471;6235036;46282;70376");
            list.Add("87;1015182;6326135;94550;74821");
            list.Add("88;1039625;6220181;61321;73730");
            list.Add("89;928260;6267217;26197;76189");
            list.Add("90;932257;6351601;30254;67071");
            list.Add("91;963129;6248129;29830;75179");
            list.Add("92;1012028;6238520;34275;70825");
            list.Add("93;951447;6355121;74521;68309");
            list.Add("94;1122436;6435918;48255;97440");
            list.Add("95;1177712;6319964;34760;91522");
            list.Add("96;1200720;6315814;73986;91564");
            list.Add("97;1199210;6288458;27000;91710");
            list.Add("98;1174191;6295782;22240;91572");
            list.Add("99;1195882;6287793;90911;91710");
            list.Add("100;1131333;6301928;65831;91625");
            list.Add("101;1217939;6268829;39190;91781");
            list.Add("102;1112561;6165909;88056;89231");
            list.Add("103;1074754;6216252;21565;73037");
            list.Add("104;1036966;6308200;91894;74196");
            list.Add("105;1035537;6293300;62267;74248");
            list.Add("106;1167544;6244180;44621;86720");
            list.Add("107;1148397;6279415;35530;91550");
            list.Add("108;1024610;6345916;29031;74838");
            list.Add("109;1108282;6435575;59457;97450");
            list.Add("110;1069367;6218623;84468;73035");
            list.Add("111;950932;6226568;93169;75323");
            list.Add("112;1079241;6363356;86814;97922");
            list.Add("113;1055424;6401813;39750;97877");
            list.Add("114;1037264;6222489;30606;73730");
            list.Add("115;989918;6459408;46161;63456");
            list.Add("116;949843;6323947;75845;68766");
            list.Add("117;1117314;6237721;80078;73457");
            list.Add("118;1028620;6228424;92063;70327");
            list.Add("119;1095019;6235358;37688;73527");
            list.Add("120;991928;6291425;47984;75031");
            list.Add("121;1082237;6214236;86842;73084");
            list.Add("122;1120088;6298644;75587;74589");
            list.Add("123;938807;6268762;38618;76137");
            list.Add("124;953048;6291241;67295;76646");
            list.Add("125;1115106;6164203;61153;89231");
            list.Add("126;999802;6213697;37632;71034");
            list.Add("127;957622;6376659;55560;64646");
            list.Add("128;1001894;6232975;39048;71229");
            list.Add("129;1158517;6172354;77451;89331");
            list.Add("130;904506;6316232;50567;67480");
            list.Add("131;912879;6282637;52635;76870");
            list.Add("132;898292;6331070;95403;67466");
            list.Add("133;921565;6274505;43260;76744");
            list.Add("134;959964;6320199;92959;69190");
            list.Add("135;943616;6370056;31307;68623");
            list.Add("136;904140;6300567;84295;76829");
            list.Add("137;904140;6300567;45338;76829");
            list.Add("138;904159;6302852;16960;76829");
            list.Add("139;1107161;6399962;90981;97074");
            list.Add("140;961614;6417006;28063;64283");
            list.Add("141;1009877;6432412;27753;63811");
            list.Add("142;994921;6458031;19088;63457");
            list.Add("143;1130025;6393117;43326;97318");
            list.Add("144;1026153;6233940;96472;70372");
            list.Add("145;1031429;6238317;61999;70736");
            list.Add("146;928770;6274240;42266;76187");
            list.Add("147;1139187;6203780;23328;89537");
            list.Add("148;1025263;6291391;74219;74072");
            list.Add("149;1024417;6234663;83474;70372");
            list.Add("150;1056713;6302092;80463;74613");
            list.Add("151;1160442;6196790;65283;89415");
            list.Add("152;945521;6254362;66588;76307");
            list.Add("153;1059243;6219430;53436;73061");
            list.Add("154;1110325;6166559;32684;89073");
            list.Add("155;1008214;6281377;78649;74336");
            list.Add("156;953438;6287861;31242;76646");
            list.Add("157;979030;6256979;92656;75417");
            list.Add("158;1108502;6172656;45912;89081");
            list.Add("159;929140;6366924;24681;67240");
            list.Add("160;1024312;6306911;44130;74177");
            list.Add("161;1065284;6439429;52591;97816");
            list.Add("162;1090712;6202544;76127;73312");
            list.Add("163;1095233;6205001;96542;73312");
            list.Add("164;1071863;6218335;38473;73033");
            list.Add("165;1085700;6352252;47461;97980");
            list.Add("166;1119242;6401217;38108;97337");
            list.Add("167;1026244;6387861;73579;63897");
            list.Add("168;1168198;6243820;84850;86720");
            list.Add("169;972354;6454348;24419;63069");
            list.Add("170;1133483;6331762;24424;91541");
            list.Add("171;1057444;6398216;90011;97877");
            list.Add("172;991144;6461461;87381;63450");
            list.Add("173;1063691;6219266;22590;73066");
            list.Add("174;1012514;6238587;91617;70825");
            list.Add("175;1054602;6300764;24411;74613");
            list.Add("176;1030457;6262870;58929;71711");
            list.Add("177;907714;6326817;17994;67433");
            list.Add("178;963567;6345329;29185;69221");
            list.Add("179;929860;6354568;40431;67227");
            list.Add("180;954611;6344637;23518;68535");
            list.Add("181;940457;6361244;81109;68307");
            list.Add("182;1016664;6187986;84704;72138");
            list.Add("183;929909;6340174;74172;67112");
            list.Add("184;928741;6377679;82492;67549");
            list.Add("185;912527;6348038;28642;67098");
            list.Add("186;953567;6333297;58069;68723");
            list.Add("187;1058142;6397471;58375;97877");
            list.Add("188;1000706;6385138;83748;64720");
            list.Add("189;955198;6392589;20844;64665");
            list.Add("190;1184800;6298389;50937;91737");
            list.Add("191;986084;6438123;84483;63110");
            list.Add("192;964053;6431516;72642;63329");
            list.Add("193;955227;6415308;97462;64347");
            list.Add("194;958281;6376539;90621;64646");
            list.Add("195;978678;6447370;91752;63150");
            list.Add("196;1097157;6399514;41040;97204");
            list.Add("197;1004121;6465304;57639;63517");
            list.Add("198;1092381;6404448;46037;97265");
            list.Add("199;1157031;6354507;38277;91438");
            list.Add("200;952948;6357656;82779;68519");
            list.Add("201;1072969;6218579;37806;73033");
            list.Add("202;1077706;6211942;40667;73037");
            list.Add("203;917665;6252520;35132;76467");
            list.Add("204;1069213;6318983;64372;74676");
            list.Add("205;946238;6247917;86454;76307");
            list.Add("206;1074157;6375110;88336;97941");
            list.Add("207;1085729;6432450;29998;97753");
            list.Add("208;1115471;6402819;47283;97228");
            list.Add("209;1047657;6340789;71635;74706");
            list.Add("210;1109176;6403871;18473;97076");
            list.Add("211;1131213;6248465;30517;73463");
            list.Add("212;935032;6258700;77340;76275");
            list.Add("213;931160;6266357;98000;76135");
            list.Add("214;984349;6461624;29540;63477");
            list.Add("215;960018;6414568;23624;64295");
            list.Add("216;967534;6444767;97421;63303");
            list.Add("217;986528;6225409;74734;71263");
            list.Add("218;1165775;6241988;44247;86720");
            list.Add("219;1072258;6222004;74162;73098");
            list.Add("220;948373;6304059;89316;68753");
            list.Add("221;1025460;6184807;79375;72766");
            list.Add("222;1016804;6217684;52311;70771");
            list.Add("223;1018299;6235519;50542;70469");
            list.Add("224;1039894;6221215;55263;73730");
            list.Add("225;1004203;6217757;16681;71065");
            list.Add("226;922546;6274400;64057;76744");
            list.Add("227;1116524;6155165;87855;89250");
            list.Add("228;1017184;6250732;81604;71679");
            list.Add("229;1058795;6235980;34096;73614");
            list.Add("230;962816;6359474;76251;69469");
            list.Add("231;1031280;6336981;76643;74850");
            list.Add("232;1118078;6289774;44585;74564");
            list.Add("233;1031249;6223919;33794;73733");
            list.Add("234;955116;6406450;74668;64319");
            list.Add("235;1122652;6248528;32943;73460");
            list.Add("236;1043439;6236404;86958;71384");
            list.Add("237;1041212;6236417;61855;71384");
            list.Add("238;1040217;6235196;86692;71384");
            list.Add("239;959786;6245975;70602;75217");
            list.Add("240;1077313;6216574;54189;73054");
            list.Add("241;1074999;6217954;15386;73033");
            list.Add("242;938774;6324490;31478;67346");
            list.Add("243;1029038;6218267;56269;73760");
            list.Add("244;1107738;6166166;28061;89077");
            list.Add("245;1219728;6273850;74275;91781");
            list.Add("246;1078784;6182238;34089;89150");
            list.Add("247;1152770;6258032;49247;73485");
            list.Add("248;944824;6344411;48442;68199");
            list.Add("249;1078496;6285432;64380;74523");
            list.Add("250;944011;6352789;72644;68167");
            list.Add("251;1178303;6275115;71962;91717");
            list.Add("252;1048393;6209638;97182;73230");
            list.Add("253;1030982;6249258;75699;71686");
            list.Add("254;1027168;6387961;73997;63897");
            list.Add("255;1070665;6214814;48624;73035");
            list.Add("256;1114126;6204509;25584;89547");
            list.Add("257;1138716;6360100;88229;97215");
            list.Add("258;1205530;6310702;88933;91575");
            list.Add("259;1109450;6406604;60182;97076");
            list.Add("260;956963;6405473;15721;64319");
            list.Add("261;1168607;6197870;32320;89407");
            list.Add("262;1086331;6434773;88912;97753");
            list.Add("263;933266;6332816;88211;67105");
            list.Add("264;1182641;6366396;63368;91413");
            list.Add("265;1025365;6297129;22408;74172");
            list.Add("266;948667;6325145;66768;68766");
            list.Add("267;930478;6267973;20020;76185");
            list.Add("268;1165207;6243796;36582;86720");
            list.Add("269;1146968;6162678;66440;89335");
            list.Add("270;1122262;6241897;42594;73430");
            list.Add("271;1164933;6196705;56158;89407");
            list.Add("272;1000913;6233503;55847;71229");
            list.Add("273;1125273;6181699;53879;89129");
            list.Add("274;1098665;6401799;42128;97204");
            list.Add("275;1029837;6277516;24220;74360");
            list.Add("276;986220;6199914;94419;71083");
            list.Add("277;1107323;6404012;29185;97076");
            list.Add("278;968924;6447302;65343;63263");
            list.Add("279;944555;6348003;30733;68165");
            list.Add("280;988081;6448669;47596;63110");
            list.Add("281;1041264;6365434;19405;74731");
            list.Add("282;944896;6346346;44249;68199");
            list.Add("283;908415;6363212;53890;67269");
            list.Add("284;965515;6326751;61089;69181");
            list.Add("285;1130260;6260124;20906;73479");
            list.Add("286;1025524;6287837;65934;74074");
            list.Add("287;1177312;6366444;72990;91413");
            list.Add("288;1025480;6232252;15959;70***");
            list.Add("289;1026447;6239173;49705;70378");
            list.Add("290;1023090;6218859;71830;70599");
            list.Add("291;1024399;6222900;79662;70619");
            list.Add("292;1020945;6244580;92825;70806");
            list.Add("293;1046046;6179589;54162;72574");
            list.Add("294;1022916;6183167;95650;72760");
            list.Add("295;1049220;6257084;38282;71522");
            list.Add("296;1073594;6217935;89961;73033");
            list.Add("297;1077706;6211942;63099;73037");
            list.Add("298;1079534;6217121;88993;73054");
            list.Add("299;1078413;6216218;96602;73054");
            list.Add("300;1089721;6214010;39536;73072");
            list.Add("301;1015259;6261322;57639;74321");
            list.Add("302;1163182;6263053;73242;86742");
            list.Add("303;1030486;6220714;73896;73760");
            list.Add("304;957035;6421966;22487;64331");
            list.Add("305;1196128;6286528;85226;91710");
            list.Add("306;951178;6336091;91232;68782");
            list.Add("307;1084970;6212937;86380;73079");
            list.Add("308;1064173;6216906;79869;73095");
            list.Add("309;1074152;6224870;53904;73102");
            list.Add("310;1071205;6207372;28147;73105");
            list.Add("311;1059517;6212793;30319;73278");
            list.Add("312;1094666;6204906;56717;73312");
            list.Add("313;1121762;6249746;77753;73460");
            list.Add("314;1086715;6231844;90406;73525");
            list.Add("315;1060369;6236581;74649;73614");
            list.Add("316;1035556;6223008;41829;73728");
            list.Add("317;1006915;6261664;93280;74343");
            list.Add("318;1016510;6267185;65763;74354");
            list.Add("319;1109401;6301193;41272;74592");
            list.Add("320;1036037;6354573;26791;74722");
            list.Add("321;967017;6243561;27738;75181");
            list.Add("322;971204;6248327;42270;75181");
            list.Add("323;953051;6229920;49507;75323");
            list.Add("324;961480;6232406;86713;75328");
            list.Add("325;974705;6218083;66520;75365");
            list.Add("326;935219;6268536;42254;76137");
            list.Add("327;1007834;6291420;97997;74193");
            list.Add("328;960810;6284495;83740;76646");
            list.Add("329;914231;6244263;59705;76437");
            list.Add("330;1013706;6435667;64979;63741");
            list.Add("331;1115457;6403396;41218;97228");
            list.Add("332;1011162;6307868;53930;74906");
            list.Add("333;927913;6371662;81599;67547");
            list.Add("334;1000869;6382820;21538;64720");
            list.Add("335;1140703;6386890;75558;97346");
            list.Add("336;947936;6241751;46073;75334");
            list.Add("337;903404;6302289;21693;76829");
            list.Add("338;1143243;6175886;82533;89312");
            list.Add("339;1148397;6295272;93668;91555");
            list.Add("340;1023090;6249105;18668;71638");
            list.Add("341;1176119;6319377;67429;91522");
            list.Add("342;930453;6306604;57222;76726");
            list.Add("343;1113591;6165722;30036;89231");
            list.Add("344;916751;6326923;57134;67454");
            list.Add("345;986013;6461635;43230;63477");
            list.Add("346;1026583;6234698;72846;70327");
            list.Add("347;1197508;6220205;48501;86609");
            list.Add("348;1076971;6314962;30019;74653");
            list.Add("349;1037423;6188828;64222;72581");
            list.Add("350;933545;6258416;43995;76275");
            list.Add("351;1002206;6212683;52068;71032");
            list.Add("352;1017135;6273601;85249;74366");
            list.Add("353;998590;6214366;67198;71034");
            list.Add("354;1034046;6237086;82502;71332");
            list.Add("355;1008882;6234720;34131;70839");
            list.Add("356;1039558;6201703;90950;72622");
            list.Add("357;1131011;6217204;29875;89520");
            list.Add("358;1030842;6258397;70666;71672");
            list.Add("359;1116481;6291987;47640;74564");
            list.Add("360;1030347;6236120;22168;70736");
            list.Add("361;1131545;6396927;53213;97318");
            list.Add("362;1121017;6291719;62839;74564");
            list.Add("363;1025090;6214265;24761;70794");
            list.Add("364;930076;6376259;88265;67547");
            list.Add("365;979919;6418416;89722;64846");
            list.Add("366;904140;6291044;88271;76865");
            list.Add("367;950634;6283316;65354;76646");
            list.Add("368;979231;6279420;79486;75038");
            list.Add("369;1043866;6247792;32186;71364");
            list.Add("370;920906;6376178;59095;67551");
            list.Add("371;919117;6330276;25627;67454");
            list.Add("372;1041553;6240102;67870;71404");
            list.Add("373;1001101;6278516;37194;74363");
            list.Add("374;1022181;6256887;87072;71691");
            list.Add("375;950682;6263159;43629;76327");
            list.Add("376;931886;6257321;39793;76275");
            list.Add("377;963783;6363443;61564;69469");
            list.Add("378;956535;6349650;59571;68526");
            list.Add("379;1023594;6188226;60587;72768");
            list.Add("380;1059666;6247975;25832;73635");
            list.Add("381;933475;6269270;49845;76133");
            list.Add("382;910239;6300919;50957;76877");
            list.Add("383;931819;6275409;61217;76149");
            list.Add("384;975927;6253559;85101;75223");
            list.Add("385;946805;6287307;68036;76297");
            list.Add("386;975990;6455112;34291;63071");
            list.Add("387;983117;6451130;81193;63179");
            list.Add("388;964923;6435639;58243;63225");
            list.Add("389;967939;6439813;35062;63303");
            list.Add("390;964384;6418348;77754;64287");
            list.Add("391;962417;6415983;53516;64283");
            list.Add("392;961153;6418079;79650;64293");
            list.Add("393;961893;6417146;35286;64283");
            list.Add("394;971046;6409397;58249;64372");
            list.Add("395;999200;6466228;27972;63526");
            list.Add("396;965586;6332787;55371;69126");
            list.Add("397;1019679;6433157;97593;63743");
            list.Add("398;973975;6454507;77502;63069");
            list.Add("399;971493;6220166;94992;75365");
            list.Add("400;928674;6350851;70414;67071");
            list.Add("401;1090039;6234403;65235;73525");
            list.Add("402;939957;6349254;82341;67061");
            list.Add("403;1108046;6166829;18470;89077");
            list.Add("404;983382;6418690;30056;64807");
            list.Add("405;908810;6327212;56665;67433");
            list.Add("406;1101091;6409623;85763;97209");
            list.Add("407;1025090;6214265;91731;70794");
            list.Add("408;984791;6452135;80476;63179");
            list.Add("409;971493;6451698;43432;63069");
            list.Add("410;897830;6363913;26285;67304");
            list.Add("411;934825;6344464;84095;67065");
            list.Add("412;1088697;6286519;49268;74523");
            list.Add("413;999205;6348375;88067;69412");
            list.Add("414;962200;6336889;27908;69123");
            list.Add("415;1063691;6219266;38560;73066");
            list.Add("416;1015495;6258980;91263;74321");
            list.Add("417;1037841;6357917;25264;74722");
            list.Add("418;1195406;6289059;58027;91710");
            list.Add("419;1014737;6242287;75500;70825");
            list.Add("420;1125389;6182281;80464;89129");
            list.Add("421;1182742;6367012;66109;91413");
            list.Add("422;1133079;6331921;55129;91541");
            list.Add("423;986081;6310460;97095;74889");
            list.Add("424;1042718;6212308;63655;73240");
            list.Add("425;1086403;6349451;87760;97980");
            list.Add("426;1082213;6287783;22940;74523");
            list.Add("427;1157982;6353078;35943;91438");
            list.Add("428;947786;6348981;91130;68163");
            list.Add("429;1057598;6235753;88877;73614");
            list.Add("430;1026447;6241054;30158;70378");
            list.Add("431;1017063;6327920;60797;74821");
            list.Add("432;1130150;6380363;27812;97342");
            list.Add("433;1017501;6222268;75171;70567");
            list.Add("434;994637;6465035;88528;63452");
            list.Add("435;1000653;6382863;24324;64720");
            list.Add("436;928722;6268060;63267;76189");
            list.Add("437;1094329;6202655;64671;73312");
            list.Add("438;1022624;6293276;36002;74080");
            list.Add("439;1028029;6213697;95081;70794");
            list.Add("440;1083233;6371238;42846;97947");
            list.Add("441;940212;6347754;77136;67061");
            list.Add("442;963822;6343256;80654;69221");
            list.Add("443;1025129;6296508;96631;74076");
            list.Add("444;1221229;6272388;45943;91781");
            list.Add("445;1163807;6381128;67857;91443");
            list.Add("446;1133127;6331362;61706;91541");
            list.Add("447;1185633;6316242;51419;91623");
            list.Add("448;1179261;6275304;42225;91717");
            list.Add("449;1167592;6241704;20597;86720");
            list.Add("450;1033458;6293420;16269;74189");
            list.Add("451;1107266;6395373;88385;97084");
            list.Add("452;1051144;6393738;58500;97877");
            list.Add("453;1148527;6344113;42327;91593");
            list.Add("454;1177110;6352894;17179;91459");
            list.Add("455;1172276;6363197;71236;91463");
            list.Add("456;1048474;6439477;78654;97843");
            list.Add("457;1065947;6436793;75419;97816");
            list.Add("458;1052909;6450394;94140;97833");
            list.Add("459;965712;6330218;18203;69126");
            list.Add("460;962167;6361380;19895;69469");
            list.Add("461;928897;6360089;96938;67227");
            list.Add("462;970527;6250573;31925;75175");
            list.Add("463;964086;6251087;69140;75179");
            list.Add("464;967193;6304093;21538;76684");
            list.Add("465;928827;6269690;23584;76185");
            list.Add("466;933760;6269526;57545;76133");
            list.Add("467;1025110;6296469;37422;74076");
            list.Add("468;909666;6362645;28830;67269");
            list.Add("469;956141;6421282;54282;64331");
            list.Add("470;1113730;6165986;76569;89231");
            list.Add("471;1140703;6386890;88707;97346");
            list.Add("472;1213925;6260555;44576;91757");
            list.Add("473;1017511;6436609;30155;63741");
            list.Add("474;1068800;6405782;36931;97855");
            list.Add("475;1003846;6218060;46676;71065");
            list.Add("476;938519;6324553;19245;67346");
            list.Add("477;1122493;6239530;26459;73431");
            list.Add("478;967232;6274423;60378;75015");
            list.Add("479;1060508;6235115;21243;73614");
            list.Add("480;1108046;6166829;56907;89077");
            list.Add("481;1000774;6219002;95371;71063");
            list.Add("482;1048648;6260172;51081;71522");
            list.Add("483;1129568;6220003;89195;89520");
            list.Add("484;965274;6336716;28176;69115");
            list.Add("485;1091847;6350923;43844;97999");
            list.Add("486;1113000;6161163;40931;89231");
            list.Add("487;1127038;6213707;59127;89518");
            list.Add("488;1068679;6220483;38044;73117");
            list.Add("489;1105285;6400533;21297;97072");
            list.Add("490;1031574;6190362;38414;72555");
            list.Add("491;1038610;6205050;22272;72622");
            list.Add("492;979145;6435157;62985;63322");
            list.Add("493;1020555;6471778;27640;63571");
            list.Add("494;1012899;6222292;67027;70563");
            list.Add("495;942023;6353762;26803;68169");
            list.Add("496;956857;6382887;53033;64625");
            list.Add("497;934267;6315529;40258;67354");
            list.Add("498;950888;6346845;24454;68239");
            list.Add("499;960537;6419435;52274;64293");
            list.Add("500;1141284;6315632;29636;91583");
            list.Add("501;1020428;6236908;50804;70469");
            list.Add("502;1103813;6215174;51462;89558");
            list.Add("503;1028231;6308282;74988;74229");
            list.Add("504;1089134;6239006;94861;73557");
            list.Add("505;1025239;6290040;97764;74074");
            list.Add("506;1082525;6288313;58806;74523");
            list.Add("507;1018785;6405511;22648;63906");
            list.Add("508;1030376;6388841;70926;63927");
            list.Add("509;1016736;6439867;77719;63864");
            list.Add("510;1075908;6449630;79681;97737");
            list.Add("511;1028889;6434182;60885;63856");
            list.Add("512;1079731;6241165;32739;73553");
            list.Add("513;1106617;6401808;64734;97074");
            list.Add("514;1048277;6297648;53173;74626");
            list.Add("515;1127389;6381825;66210;97340");
            list.Add("516;1054030;6234639;41555;73650");
            list.Add("517;999037;6347244;58454;69412");
            list.Add("518;1009960;6432941;39639;63811");
            list.Add("519;1017213;6424644;50797;63834");
            list.Add("520;1036517;6396662;79411;97896");
            list.Add("521;1024816;6389732;97350;63924");
            list.Add("522;1067819;6412342;92365;97828");
            list.Add("523;1130531;6204699;38123;89542");
            list.Add("524;1008310;6437967;31457;63801");
            list.Add("525;1118953;6380944;72489;97199");
            list.Add("526;992193;6260417;48916;75428");
            list.Add("527;1054092;6255477;68871;71554");
            list.Add("528;971084;6247735;22251;75181");
            list.Add("529;964832;6317157;38203;69168");
            list.Add("530;1050129;6257470;90125;71522");
            list.Add("531;975966;6440150;91630;63128");
            list.Add("532;1014038;6221292;29942;70565");
            list.Add("533;999937;6445777;72744;63500");
            list.Add("534;996786;6446758;65287;63500");
            list.Add("535;986956;6445893;56636;63110");
            list.Add("536;1006189;6468219;43117;63505");
            list.Add("537;992270;6461673;86640;63450");
            list.Add("538;958059;6313496;47701;68789");
            list.Add("539;946824;6355940;35968;68309");
            list.Add("540;1004534;6470715;97575;63505");
            list.Add("541;1018467;6252318;44754;71634");
            list.Add("542;964870;6438197;89290;63225");
            list.Add("543;960781;6400260;71669;64342");
            list.Add("544;1056766;6268463;63641;71560");
            list.Add("545;960570;6396576;63959;64342");
            list.Add("546;943184;6368689;39365;68623");
            list.Add("547;1040808;6241222;26167;71404");
            list.Add("548;992876;6417016;96878;64823");
            list.Add("549;982362;6430156;82233;64859");
            list.Add("550;961311;6413746;81974;64285");
            list.Add("551;956963;6405473;73845;64319");
            list.Add("552;919192;6328437;92504;67454");
            list.Add("553;998143;6453708;41508;63538");
            list.Add("554;959228;6382171;35586;64625");
            list.Add("555;983392;6459961;26850;63165");
            list.Add("556;955540;6427001;55233;64331");
            list.Add("557;956141;6421282;34160;64331");
            list.Add("558;964923;6342261;81835;69121");
            list.Add("559;954876;6348181;47394;68535");
            list.Add("560;940365;6280444;28783;76297");
            list.Add("561;992799;6465362;38965;63452");
            list.Add("562;986013;6461635;65435;63477");

            return list;
        }
    }
}
