﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{
    /**
     * Customized implementation of the algorithm, to compute Chi-Shapes, presented in http://www.geosensor.net/papers/duckham08.PR.pdf
     * Calculates the Chi-Shape of a point set, by using the delaunay triangulation and an input factor ([0,1])
     * The input factor is used to calculate the length parameter of the algorithm as a fraction of the maximum edge length
    **/
    class ChiAlgorithm
    {
        private ChiTriangulation chiTriangles { get; set; }
        private double l { get; set; }
        private double maxEdgeLength { get; set; }

        public ChiAlgorithm()
        {
            chiTriangles = new ChiTriangulation();
            l = 0;
        }

        public ChiTriangulation runAlgorithm(Triangulation delaunay, double factor)
        {
            prepareAlgorithm(delaunay);
            l = factor * maxEdgeLength;
            bool finished = false;
            while (!finished)
            {
                int counter = 0;
                // Loops over external edges, to find a candidate for removal
                foreach (Edge e in chiTriangles.externalEdges)
                {
                    // If the edge is too short, the algorithm is finished, as all following edges must be shorter, thus no edge can be removed
                    if (e.getLength() <= l)
                    {
                        finished = true;
                        break;
                    }
                    else
                    {
                        // Check if edge was already considered
                        if (!e.chiFinished)
                        {
                            // Get the point opposite of the edge in the triangle
                            Point third = e.triangles[0].getThirdPointByID(e);
                            
                            // If the third point in the triangle is external, the edge cannot be removed, as that would destroy the polygon
                            if (chiTriangles.externalPoints.ContainsKey(third.ID))
                            {
                                // Set edge to finished
                                e.finish();
                                // If this was the last external edge, the algorithm is finished
                                if (counter == (chiTriangles.externalEdges.Count - 1))
                                {
                                    finished = true;
                                    break;
                                }
                            }
                            else
                            {
                                // Add the other two edges in the triangle to the external edges, add new external point
                                foreach (Edge e2 in e.triangles[0].edges)
                                {
                                    // Get edge by id from list in ChiTriangulation, as only here the triangles are correct
                                    Edge e3 = chiTriangles.edges[e2.ID];
                                    // Check, if the edge is not the one used for iteration, we only want the other two edges
                                    if (!e.isEqual(e3))
                                    {
                                        // Remove the triangle with e in it from the triangles of e3, it no longer exists, when e is removed
                                        int i = 0;
                                        int index = 0;
                                        foreach (Triangle t in e3.triangles)
                                        {
                                            if (t.ID == e.triangles[0].ID)
                                            {
                                                index = i;
                                            }
                                            i++;
                                        }
                                        e3.triangles.RemoveAt(index);

                                        // Find index, at which e3 is inserted into the sorted list of edges and insert it
                                        int idx = chiTriangles.externalEdges.BinarySearch(e3, new ByEdgeLength());
                                        if (idx < 0) idx = ~idx;
                                        chiTriangles.externalEdges.Insert(idx, e3);

                                        // Add point to external points
                                        if (!chiTriangles.externalPoints.ContainsKey(e3.P.ID))
                                        {
                                            chiTriangles.externalPoints.Add(e3.P.ID, e3.P);
                                        }
                                        if (!chiTriangles.externalPoints.ContainsKey(e3.Q.ID))
                                        {
                                            chiTriangles.externalPoints.Add(e3.Q.ID, e3.Q);
                                        }
                                    }
                                }
                                
                                // Remove edge e and the corresponding triangle
                                chiTriangles.externalEdges.Remove(e);
                                chiTriangles.triangles.Remove(e.triangles[0].ID);

                                // Break the loop, iteration has to be restarted, as the newly inserted edges could have been inserted before the current iteration point
                                break;
                            }
                        }
                    }
                    // Increment edge counter, used for abortion
                    counter++;
                }
            }
            return chiTriangles;
        }

        // Preprocesses the input data
        private void prepareAlgorithm(Triangulation delaunay)
        {
            // Add all triangles and edges to the ChiTriangulation, creates a list of edge IDs
            List<int> edgeList = new List<int>();
            foreach (Triangle tri in delaunay.triangles)
            {
                chiTriangles.triangles.Add(tri.ID, tri);
                foreach (Edge e in tri.edges)
                {
                    if (!chiTriangles.edges.ContainsKey(e.ID))
                    {
                        chiTriangles.edges.Add(e.ID, e);
                        edgeList.Add(e.ID);
                    }
                    // Adds the triangle to the edge, this will be used to check, if edges are external
                    chiTriangles.edges[e.ID].triangles.Add(tri);
                }
            }
            // Gets the max edge length, constructs external points/edges lists
            foreach (int eID in edgeList)
            {
                Edge e = chiTriangles.edges[eID];
                if (e.getLength() > maxEdgeLength)
                    maxEdgeLength = e.getLength();
                // If an edge is only part of one triangle, it is external
                if (e.triangles.Count == 1)
                {
                    chiTriangles.externalEdges.Add(e);
                    if (!chiTriangles.externalPoints.ContainsKey(e.P.ID))
                    {
                        chiTriangles.externalPoints.Add(e.P.ID, e.P);
                    }
                    if (!chiTriangles.externalPoints.ContainsKey(e.Q.ID))
                    {
                        chiTriangles.externalPoints.Add(e.Q.ID, e.Q);
                    }
                }
            }
            // Sort external edges by length
            chiTriangles.externalEdges.Sort(new ByEdgeLength());
        }
    }

    // Compares two edges by their length, used to sort the edges
    class ByEdgeLength : IComparer<Edge>
    {
        public int Compare(Edge x, Edge y)
        {
            if (x.getLength() < y.getLength())
            {
                return 1;
            }
            else if (x.getLength() > y.getLength())
            {
                return -1;
            }
            else
            {
                return 0;
            }
        }
    }
}
