﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace VisualizingChiShapes
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Territories territories;
        public MainWindow()
        {
            InitializeComponent();
            l_slider.Value = 0.7;
            showDelaunay.IsChecked = true;
            runSample();
        }

        /**
         * This sample will use PTV xTerritory, to calculate 3 territories for a set of locations with one activity measure.
         * The territories will subsequently be visualized, using PTV xMap.
         * In this case Characteristic shapes (Chi-shapes) (http://www.geosensor.net/phpws/index.php?module=pagemaster&PAGE_user_op=view_printable&PAGE_id=13&lay_quiet=1) 
         * and Delaunay triangulations (http://en.wikipedia.org/wiki/Delaunay_triangulation) 
         * will be used to visualize territories.
         * 
         * Characteristic shapes are one method, to obtain a "concave hull" of a set of points. This is desirable, because in many cases
         * the convex hull of a set of points does not appropriately represent the shape formed by the points.
        **/
        private void runSample()
        {
            territories = calculateTerritories();
            if (territories != null)
                mapTerritories(ref territories);

        }

        private Territories calculateTerritories()
        {
            //Step 1: Initiate xTerritory client
            XTerritoryWSService xTerritoryClient = new XTerritoryWSService();

            //Step 2: Create CallerContext with appropriate profile and coordinate format
            CallerContext context = new CallerContext();
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            context.wrappedProperties = properties;

            //Step 3: Load the locations for territory planning
            List<String> locationStrings = getLocations();
            Location[] locations = new Location[locationStrings.Count];
            for (int i = 0; i < locationStrings.Count; i++)
            {
                String[] splitString = locationStrings[i].Split(';');
                String locID = splitString[0];

                //Step 3.1: Load x and y coordinates as well as activity measure
                double locX = Convert.ToDouble(splitString[1]);
                double locY = Convert.ToDouble(splitString[2]);
                double locActivity = Convert.ToDouble(splitString[3]);

                xserver.Point point = new xserver.Point
                {
                    point = new PlainPoint { x = locX, y = locY }
                };

                //Step 3.2: Create a Location with the data above
                locations[i] = new Location
                {
                    id = locID,
                    coordinate = point,
                    activity = new AbsoluteActivity { value = locActivity }
                };
            }

            //Step 4: Create DistanceCalculationOptions, here a new distance matrix will be used.
            NewDistanceMatrixByRoad distOptions = new NewDistanceMatrixByRoad();

            //Step 5: Create PlanningOptions and specify the number of territories
            PlanningOptions options = new PlanningOptions
            {
                exactNumberOfTerritories = 3                //the visualization part of this sample supports a maximum of 3 territories
            };

            //Step 6: Create PlanningProfile
            PlanningProfile profile = new PlanningProfile
            {
                solutionQuality = SolutionQuality.BASIC,    //Solution quality, with larger problems this impacts the running time severely
                avoidOverlappingTerritories = true,         //If true, the algorithm tries to avoid visual overlaps between territories,
                                                            //if a solution without overlaps is possible
                createBalancedTerritories = true,           //If true, the imbalanceTolerance parameter will be considered
                imbalanceTolerance = 10.0,                  //Maximum deviation from average activity per territory
            };

            //Step 7: Create ResponseContents, specify which information the client returns
            ResponseContents contents = new ResponseContents
            {
                locations = true,
                territories = true,
            };

            try
            {
                //Step 8: Calculate territories
                Territories territories = xTerritoryClient.planTerritories(locations, distOptions, options, profile, contents, context);
                return territories;
            }
            catch (Exception e)
            {
                return null;
            }
        }

        private void mapTerritories(ref Territories territories)
        {
            //Step 1: Create xMap client
            XMapWSService client = new XMapWSService();

            //Step 2: Create ImageInfo
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 800,
                width = 1280,
                imageParameter = ""
            };

            //Step 3: Create MapParams
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

            //Step 4: Create Bitmap array, each location will be visualized with a bitmap marker
            Bitmap[] bitmapArray = new Bitmap[territories.wrappedLocations.Length];

            //Step 5: Create list of territoryIDs, it will be used to match with the territoryIDs of the locations
            List<String> territoryIDs = new List<String>();

            //Step 6: Create Dictionary with territoryID as ID and lists of locations as entries
            //        For each territory all locations will be saved, which it contains
            Dictionary<String, List<Location>> locationLists = new Dictionary<String, List<Location>>();
            foreach (Territory ter in territories.wrappedTerritories)
            {
                locationLists.Add(ter.id, new List<Location>());
                territoryIDs.Add(ter.id);
            }
            foreach (Location loc in territories.wrappedLocations)
            {
                locationLists[loc.territoryId].Add(loc);
            }

            int territoryCounter = 0;

            //Step 7: Initialize Lines array, create Options for the Delaunay triangulation
            Lines[] lines = new Lines[territories.wrappedTerritories.Length * 2];
            LineOptions lineOptionsDelaunay = new LineOptions();
            LinePartOptions partoptionsDelaunay = new LinePartOptions
            {
                visible = true,
                color = new xserver.Color { red = 125, green = 125, blue = 125 },
                width = 1,
            };
            lineOptionsDelaunay.mainLine = partoptionsDelaunay;

            //Step 8: Loop over territories
            foreach (Territory ter in territories.wrappedTerritories)
            {
                //Step 8.1: Calculate Delaunay triangulation
                Geometry.BowyerWatsonAlgorithm bw = new Geometry.BowyerWatsonAlgorithm();
                Geometry.Triangulation delaunay = bw.calculateDelaunay(locationLists[ter.id]);

                //Step 8.2: Create LineString from triangulation, add delaunay triangulation to Lines array
                int iterator = 0;
                if (showDelaunay.IsChecked == true)
                {
                    LineString[] lineStringsDelaunay = new LineString[delaunay.triangles.Count * 3];
                    foreach (Geometry.Triangle tri in delaunay.triangles)
                    {
                        foreach (Geometry.Edge e in tri.edges)
                        {
                            PlainPoint[] plainPoints = new PlainPoint[2];
                            plainPoints[0] = e.P.pt;
                            plainPoints[1] = e.Q.pt;
                            PlainLineString plainLineString = new PlainLineString { wrappedPoints = plainPoints };
                            lineStringsDelaunay[iterator] = new LineString { lineString = plainLineString };
                            iterator++;
                        }
                    }
                    lines[territoryCounter] = new Lines { wrappedLines = lineStringsDelaunay, options = lineOptionsDelaunay };
                }

                //Step 8.3: Calculate Chi-Shape
                Geometry.ChiAlgorithm chi = new Geometry.ChiAlgorithm();
                Geometry.ChiTriangulation chiShape = chi.runAlgorithm(delaunay, l_slider.Value);

                //Step 8.4: Create LineString from shape
                LineString[] lineStringsChi = new LineString[chiShape.externalEdges.Count];
                iterator = 0;
                foreach (Geometry.Edge e in chiShape.externalEdges)
                {
                    PlainPoint[] plainPoints = new PlainPoint[2];
                    plainPoints[0] = e.P.pt;
                    plainPoints[1] = e.Q.pt;
                    PlainLineString plainLineString = new PlainLineString { wrappedPoints = plainPoints };
                    lineStringsChi[iterator] = new LineString { lineString = plainLineString };
                    iterator++;
                }
                //Step 8.5: Specify LineOptions depending on the territoryID, add lines to Lines array
                LineOptions lineOptionsChi = new LineOptions();
                xserver.Color color = new xserver.Color { red = 0, green = 255, blue = 0 };

                if (ter.id == territoryIDs[1])
                    color = new xserver.Color { red = 255, green = 0, blue = 0 };
                if (ter.id == territoryIDs[2])
                    color = new xserver.Color { red = 0, green = 0, blue = 255 };

                LinePartOptions partoptionsChi = new LinePartOptions
                {
                    visible = true,
                    color = color,
                    width = 3,
                };
                lineOptionsChi.mainLine = partoptionsChi;

                lines[territoryCounter + territories.wrappedTerritories.Length] = new Lines { wrappedLines = lineStringsChi, options = lineOptionsChi };
                territoryCounter++;
            }

            //Step 9: Create line layer
            CustomLayer customLineLayer = new CustomLayer
            {
                visible = true,
                drawPriority = 25000,
                wrappedLines = lines,
                objectInfos = ObjectInfoType.NONE
            };

            //Step 10: Insert appropriate Bitmap into array for each location
            int locationCounter = 0;
            foreach (Location location in territories.wrappedLocations)
            {
                String bmp = "location.bmp";
                if (location.territoryId == territoryIDs[0])
                {
                    bmp = "flaggreen.bmp";
                }
                else if (location.territoryId == territoryIDs[1])
                {
                    bmp = "flagred.bmp";
                }
                else if (location.territoryId == territoryIDs[2])
                {
                    bmp = "flagblue.bmp";
                }
                xserver.Point point = location.coordinate;
                bitmapArray[locationCounter] = new Bitmap
                {
                    name = bmp,
                    position = point
                };
                locationCounter++;
            }

            //Step 11: Create BitmapOptions and Bitmaps[]
            BitmapOptions bitmapOptions = new BitmapOptions
            {
                referencePoint = new PlainPoint { x = 25, y = 32 },
                transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 }
            };
            Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };

            //Step 12: Create CustomLayer for bitmaps
            CustomLayer customBMPLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25002,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            //Step 13: Add custom layers to Layer array
            Layer[] layers = new Layer[2];
            layers[0] = customBMPLayer;
            layers[1] = customLineLayer;

            bool includeImageInResponse = false;

            try
            {
                // Step 14: Return map object with its characteristics
                Map map = client.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 15: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // proper exception handling: see API for PTV's exception handling!
            }
        }

        private List<String> getLocations()
        {
            List<String> list = new List<String>();

            // orderId, X, Y, activity

            list.Add("101;671006;6382294;90");
            list.Add("102;681812;6375071;120");
            list.Add("103;682676;6388154;90");
            list.Add("104;675000;6380000;60");
            list.Add("105;679143;6374455;90");
            list.Add("106;684621;6374703;180");
            list.Add("107;684659;6372947;90");
            list.Add("108;683151;6371248;30");
            list.Add("109;685669;6371954;45");
            list.Add("110;683974;6369193;60");
            list.Add("111;683000;6366903;90");
            list.Add("112;678763;6367800;120");
            list.Add("113;681397;6369670;105");
            list.Add("114;678400;6370854;90");
            list.Add("115;674779;6373188;60");
            list.Add("116;678149;6380180;45");
            list.Add("117;680440;6381326;30");
            list.Add("118;665627;6378195;90");
            list.Add("119;687472;6386122;180");
            list.Add("120;666773;6381020;120");
            list.Add("121;687136;6382563;60");
            list.Add("122;665642;6382471;30");
            list.Add("123;665627;6386365;45");
            list.Add("124;672957;6388121;60");
            list.Add("125;677004;6388121;75");
            list.Add("126;682578;6384074;90");
            list.Add("127;687540;6379111;105");
            list.Add("128;681661;6384838;120");
            list.Add("129;687388;6385754;135");
            list.Add("130;685937;6388121;150");

            return list;
        }

        private void runButton_Click(object sender, RoutedEventArgs e)
        {
            mapTerritories(ref territories);
        }
    }
}
